/*
   Copyright 2018-2019 Zephyr Software, LLC.

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.

*/

#include <ostream>
#include <set>
#include <vector>

namespace IRDB_SDK
{
	using namespace std;
	class BasicBlock_t;
	using BasicBlockSet_t = set<BasicBlock_t*>;
	using InstructionVector_t = vector<Instruction_t*>;

	class BasicBlock_t
	{
		protected:
			BasicBlock_t() {}
			BasicBlock_t(const BasicBlock_t& copy) = delete;
		public:
			virtual ~BasicBlock_t() {}

			virtual bool                       getIsExitBlock()          const = 0;
			virtual const InstructionVector_t& getInstructions()         const = 0;
			virtual const BasicBlockSet_t&     getPredecessors()         const = 0;
			virtual const BasicBlockSet_t&     getSuccessors()           const = 0;
			virtual const BasicBlockSet_t&     getIndirectTargets()      const = 0;
			virtual bool                       endsInBranch()            const = 0;
			virtual bool                       endsInIndirectBranch()    const = 0;
			virtual bool                       endsInConditionalBranch() const = 0;
			virtual Instruction_t*             getBranchInstruction()    const = 0;

			virtual void           dump(ostream &os=cout) const = 0 ;

	};
	ostream& operator<<(ostream& os, const BasicBlock_t& block);
}
