/*
   Copyright 2018-2019 Zephyr Software, LLC.

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.

*/

namespace IRDB_SDK
{

	using namespace std;

	using DominatorMap_t     = map<const BasicBlock_t*, BasicBlockSet_t>; 
	using BlockToBlockMap_t  = map<const BasicBlock_t*, const BasicBlock_t*>;
	using BasicBlockVector_t = vector<BasicBlock_t*>;

	BasicBlockVector_t getDFSOrder(ControlFlowGraph_t* cfg);

	class DominatorGraph_t
	{
		protected:
			DominatorGraph_t(){}
			DominatorGraph_t(const DominatorGraph_t& copy)=delete;
		public:
			static unique_ptr<DominatorGraph_t> factory(
			                                      const ControlFlowGraph_t* p_cfg, 
			                                      bool needs_postdoms=false, 
			                                      bool needs_idoms=false
			                                      );

			virtual const BasicBlockSet_t& getDominators(const BasicBlock_t* node)                   const = 0 ;
			virtual const BasicBlockSet_t& getDominated (const IRDB_SDK::BasicBlock_t* node)         const = 0 ;

// These interfaces to be provided in future revisions:
// 			virtual const BasicBlockSet_t& getPostDominators(const BasicBlock_t* node)               const = 0 ;
// 			virtual const BasicBlockSet_t& getPostDominated (const BasicBlock_t* node)               const = 0 ;

			virtual const BasicBlock_t*    getImmediateDominator(const BasicBlock_t* node)           const = 0 ;
                        virtual const BasicBlockSet_t& getImmediateDominated(const IRDB_SDK::BasicBlock_t* node) const = 0 ;

// These interfaces to be provided in future revisions:
// 			virtual const BasicBlock_t*    getImmediatePostDominator (const BasicBlock_t* node)      const = 0 ;
// 			virtual const BasicBlockSet_t& getImmediatePostDominated (const BasicBlock_t* node)      const = 0 ;

			virtual void                   dump(ostream &os=cout)                                    const = 0 ;
			virtual bool                   hasWarnings()                                             const = 0 ;
	
	};
	ostream& operator<<(ostream& os, const DominatorGraph_t& cfg);

}

