/*
   Copyright 2018-2019 Zephyr Software, LLC.

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.

*/

#include <irdb-core>

namespace IRDB_SDK 
{

	using namespace std;

	extern Instruction_t* insertAssemblyBefore   (FileIR_t* firp, Instruction_t* before, string assembly, Instruction_t *target=nullptr);
	extern Instruction_t* insertAssemblyAfter    (FileIR_t* firp, Instruction_t* after , string assembly, Instruction_t *target=nullptr);
/*
 * The insertAssemblyInstructionsBefore function takes in a group of (semicolon/newline-seperated) assembly statements and other template parameters, and insert them into the
 * program before the "before" instruction.
 * Param: firp -- The current FileIR_t object
 * before -- The instruction that the new instructions will be inserted before
 * templateins -- The assembly template to be inserted.  Typical assembly formatting applies:  instructions 
 * start on new lines, defined labels are allowed. For x86, the format must be in NASM style assembly.
 * templateParams -- The template arguments to be substituted into the template string.
 * Returns: A vector of the newly inserted instructions.
 * Side Effect: The new instructions are inserted into the IRDB data structures.
 */

        extern std::vector<Instruction_t*> insertAssemblyInstructionsBefore   (FileIR_t* firp, Instruction_t* before, string templateIns, vector<string> templateParams);
/*
 * The insertAssemblyInstructionsAfter function takes in a group of (semicolon/newline-seperated) assembly statements and other template parameters, and insert them into the
 * program after the "after" instruction.
 * Param: firp -- The current FileIR_t object
 * after -- The instruction that the new instructions will be inserted after
 * templateins -- The assembly template to be inserted.  Typical assembly formatting applies:  instructions 
 * start on new lines, defined labels are allowed. For x86, the format must be in NASM style assembly.
 * templateParams -- The template arguments to be substituted into the template string.
 * Returns: A vector of the newly inserted instructions.
 * Side Effect: The new instructions are inserted into the IRDB data structures.
 */
        extern std::vector<Instruction_t*> insertAssemblyInstructionsAfter    (FileIR_t* firp, Instruction_t* after , string templateIns, vector<string> templateParams);
	extern Instruction_t* insertDataBitsBefore   (FileIR_t* firp, Instruction_t* before, string dataBits, Instruction_t *target=nullptr);
	extern Instruction_t* insertDataBitsAfter    (FileIR_t* firp, Instruction_t* after , string dataBits, Instruction_t *target=nullptr);

	extern Instruction_t* addNewDataBits         (FileIR_t* firp, string p_bits);
	extern Instruction_t* addNewAssembly         (FileIR_t* firp, string p_asm);
	extern Instruction_t* addNewDataBits         (FileIR_t* firp, Instruction_t* p_after, string p_bits);
	extern Instruction_t* addNewAssembly         (FileIR_t* firp, Instruction_t* p_after, string p_asm);

	void extern setInstructionAssembly(FileIR_t* firp, Instruction_t* instr, const string& p_asm);
	extern void setInstructionAssembly(
			FileIR_t* firp, 
			Instruction_t* instr, 
			const string& p_asm, 
			Instruction_t *p_new_fallthrough, 
			Instruction_t* p_new_target=nullptr);

}
